@@ -5,13 +5,18 @@ import gzip
 import zipfile
 import bz2
 import io
+import os
 import functools
 import sys
 import numpy as np
 import pandas as pd
 import collections
 import subprocess as sp
+import re
+from os.path import join, exists
+from random import getrandbits
 from scipy.sparse import coo_matrix
+from Bio import SeqIO
 import hicstuff.hicstuff as hcs
 from hicstuff.log import logger
 
@@ -22,7 +27,7 @@ DEFAULT_INFO_CONTIGS_FILE_NAME = "info_contigs.txt"
 DEFAULT_SPARSE_MATRIX_FILE_NAME = "abs_fragments_contacts_weighted.txt"
 
 
-def raw_cols_to_sparse(sparse_array, shape=None, dtype=np.float64):
+def _cols_to_sparse(sparse_array, shape=None, dtype=np.float64):
     """
     Make a coordinate based sparse matrix from columns.
     Convert (3, n) shaped arrays to a sparse matrix. The fist
@@ -49,20 +54,21 @@ def raw_cols_to_sparse(sparse_array, shape=None, dtype=np.float64):
         >>> row, col = np.array([1, 2, 3]), np.array([3, 2, 1])
         >>> data = np.array([4, 5, 6])
         >>> M = np.array([row, col, data]).T
-        >>> S = raw_cols_to_sparse(M)
+        >>> S = _cols_to_sparse(M)
         >>> print(S.todense())
         [[0. 0. 0. 0.]
          [0. 0. 0. 4.]
          [0. 0. 5. 0.]
          [0. 6. 0. 0.]]
     """
-    if shape is None:
-        n = int(np.amax(sparse_array[:, :-1]) + 1)
-        shape = (n, n)
 
     row = sparse_array[:, 0]
     col = sparse_array[:, 1]
     data = sparse_array[:, 2]
+    if shape is None:
+        n = int(max(np.amax(row), np.amax(col))) + 1
+        shape = (n, n)
+
     S = coo_matrix((data, (row, col)), shape=shape, dtype=dtype)
     return S
 
@@ -92,13 +98,25 @@ def load_sparse_matrix(mat_path, binning=1, dtype=np.float64):
     -------
     sparse_mat : scipy.sparse.coo_matrix
         The output (sparse) matrix in COOrdinate format.
+
+    Examples
+    --------
+    >>> S = load_sparse_matrix('test_data/mat_5kb.tsv', binning=1)
+    >>> S.data[:10]
+    array([84.,  2.,  3.,  2.,  1.,  1., 50.,  1., 66.,  1.])
+    >>> S.shape
+    (16, 16)
     """
-    raw_mat = np.loadtxt(mat_path, delimiter="\t", dtype=dtype)
+    try:
+        cols_arr = np.loadtxt(mat_path, delimiter="\t", dtype=dtype)
+        shape = (int(cols_arr[0, 0]), int(cols_arr[0, 1]))
+    except ValueError:
+        cols_arr = np.loadtxt(mat_path, delimiter="\t", dtype=dtype, skiprows=1)
+        shape = None
 
     # Get values into an array without the header. Use the header to give size.
-    sparse_mat = raw_cols_to_sparse(
-        raw_mat[1:, :], shape=(int(raw_mat[0, 0]), int(raw_mat[0, 1])), dtype=dtype
-    )
+    sparse_mat = _cols_to_sparse(cols_arr[1:, :], shape=shape, dtype=dtype)
+
     if binning == "auto":
         num_bins = max(sparse_mat.shape) + 1
         subsampling_factor = num_bins // DEFAULT_MAX_MATRIX_SHAPE
@@ -120,6 +138,7 @@ def save_sparse_matrix(s_mat, path):
     path : str
         File path where the matrix will be stored
     """
+    s_mat = s_mat.tocoo()
     sparse_arr = np.vstack([s_mat.row, s_mat.col, s_mat.data]).T
 
     np.savetxt(
@@ -151,6 +170,11 @@ def load_pos_col(path, colnum, header=1, dtype=np.int64):
     -------
     numpy.array :
         A 1D numpy array with the
+
+    Examples
+    --------
+    >>> load_pos_col('test_data/mat_5kb.tsv', 0)[:10]
+    array([0, 0, 0, 0, 0, 0, 1, 1, 2, 2])
     """
     pos_arr = np.genfromtxt(
         path, delimiter="\t", usecols=(colnum,), skip_header=header, dtype=dtype
@@ -158,6 +182,159 @@ def load_pos_col(path, colnum, header=1, dtype=np.int64):
     return pos_arr
 
 
+def generate_temp_dir(path):
+    """Temporary directory generation
+
+    Generates a temporary file with a random name at the input path.
+    
+    Parameters
+    ----------
+    path : str
+        The path at which the temporary directory will be created.
+    
+    Returns
+    -------
+    str
+        The path of the newly created temporary directory.
+    """
+    exist = True
+    while exist:
+        # Keep trying random directory names if they already exist
+        directory = str(hex(getrandbits(32)))[2:]
+        full_path = join(path, directory)
+        if not exists(full_path):
+            exist = False
+    try:
+        os.makedirs(full_path)
+    except PermissionError:
+        raise PermissionError(
+            "The temporary directory cannot be created in {}. "
+            "Make sure you have write permission.".format(path)
+        )
+    return full_path
+
+
+def _check_cooler(fun):
+    """Decorates function `fun` to check if cooler is available.."""
+
+    @functools.wraps(fun)
+    def wrapped(*args, **kwargs):
+        try:
+            import cooler
+            fun.__globals__["cooler"] = cooler
+        except ImportError:
+            print(
+                "The cooler package is required to use {0}, please install it first".format(
+                    fun.__name__
+                )
+            )
+            raise
+        return fun(*args, **kwargs)
+
+    return wrapped
+
+
+@_check_cooler
+def add_cool_column(clr, column, column_name, table_name="bins", metadata={}, dtype=None):
+    """
+    Adds a new column to a loaded Cooler store. If the column exists,
+    it is replaced. This will affect the .cool file.
+    
+    Parameters
+    ----------
+    clr : Cooler object
+        A Cooler store.
+    column : pandas Series
+        The column to add to the cooler.
+    column_name : str
+        The name of the column to add.
+    table_name : str
+        The name of the table to which the column should be added.
+        Defaults to the "bins" table.
+    metadata : dict
+        A dictionary of metadata to associate with the new column.
+    """
+    with clr.open("r+") as c:
+        if column_name in c[table_name]:
+            del c[table_name][column_name]
+        h5opts = dict(compression="gzip", compression_opts=6)
+        c[table_name].create_dataset(column_name, data=column, dtype=dtype, **h5opts)
+        c[table_name][column_name].attrs.update(metadata)
+
+
+@_check_cooler
+def load_cool(cool):
+    """
+    Reads a cool file into memory and parses it into GRAAL style tables.
+    
+    Parameters
+    ----------
+    cool : str
+        Path to the input .cool file.
+
+    Returns
+    -------
+    mat : scipy coo_matrix
+        Hi-C contact map in COO format.
+    frags : pandas DataFrame
+        Table off bins matching the matrix. Corresponds to the content of the fragments_list.txt file.
+    chroms : pandas DataFrame
+        Table of chromosome informations.
+    """
+    c = cooler.Cooler(cool)  #pylint: disable=undefined-variable
+    frags = c.bins()[:]
+    chroms = c.chroms()[:]
+    mat = c.pixels()[:]
+    frags.rename(columns={"start": "start_pos", "end": "end_pos"}, inplace=True)
+    frags['id'] = frags.groupby('chrom', sort=False).cumcount() + 1
+    # Try loading hicstuff-specific columns
+    try:
+        frags = frags[['id', 'chrom', 'start_pos', 'end_pos', 'size', 'gc_content']]
+    # If absent, only load standard columns
+    except KeyError:
+        frags = frags[['id', 'chrom', 'start_pos', 'end_pos']]
+        
+    chroms['cumul_length'] = chroms.length.shift(1).fillna(0).cumsum().astype(int)
+    n_frags = c.bins()[:].groupby('chrom', sort=False).count().start
+    chroms['n_frags'] = chroms.merge(n_frags, right_index=True, left_on='name', how='left').start
+    chroms.rename(columns={'name': 'contig'}, inplace=True)
+    n = int(max(np.amax(mat.bin1_id), np.amax(mat.bin2_id))) + 1
+    shape = (n, n)
+    mat = coo_matrix((mat['count'], (mat.bin1_id, mat.bin2_id)), shape=shape)
+
+    return mat, frags, chroms
+
+
+@_check_cooler
+def save_cool(cool_out, mat, frags, metadata={}):
+    """
+    Writes a .cool file from GRAAL style tables.
+    
+    Parameters
+    ----------
+    cool_out : str
+        Path to the output cool file.
+    mat : scipy coo_matrix
+        The Hi-C contact matrix in sparse COO format.
+    frags : pandas DataFrame
+        The graal style 'fragments_list' table.
+    metadata : dict
+        Potential metadata to associate with the cool file.
+    """
+    up_tri = False
+    # Check if symmetric matrix is symmetric
+    # (i.e. only upper triangle or full mat)
+    if (abs(mat - mat.T) > 1e-10).nnz != 0:
+        up_tri = True
+    # Drop useless column
+    bins = frags.drop("id", axis=1)
+    # Get column names right
+    bins.rename(columns={"seq": "chrom", "start_pos": "start", "end_pos": "end"}, inplace=True)
+    mat_dict = {"bin1_id": mat.row, "bin2_id": mat.col, "count": mat.data}
+    pixels = pd.DataFrame(mat_dict)
+    cooler.create_cooler(cool_out, bins, pixels, metadata=metadata, symmetric_upper=up_tri, triucheck=False)  #pylint: disable=undefined-variable
+
+
 def read_compressed(filename):
     """Read compressed file
 
@@ -167,10 +344,12 @@ def read_compressed(filename):
     ----------
     filename : str
         The path to the input file
+
     Returns
     -------
     file-like object
         The handle to access the input file's content
+
     """
 
     # Standard header bytes for diff compression formats
@@ -199,7 +378,7 @@ def read_compressed(filename):
     if comp == "gz":
         return gzip.open(filename, "rt")
     elif comp == "bz2":
-        return bz2.BZ2File(filename, "rt")
+        return bz2.open(filename, "rt")
     elif comp == "zip":
         zip_arch = zipfile.ZipFile(filename, "r")
         if len(zip_arch.namelist()) > 1:
@@ -207,7 +386,7 @@ def read_compressed(filename):
         else:
             # ZipFile opens as bytes by default, using io to read as text
             zip_content = zip_arch.open(zip_arch.namelist()[0], "r")
-            return io.TextIOWrapper(zip_content)
+            return io.TextIOWrapper(zip_content, encoding="utf-8")
     else:
         return open(filename, "r")
 
@@ -379,6 +558,7 @@ def load_from_redis(key):
     ----------
     key : str
         The key of the dataset that was stored in redis.
+
     Returns
     -------
     M : numpy.ndarray
@@ -497,7 +677,7 @@ def dade_to_GRAAL(
             fragments_list.write(line_to_write)
 
 
-def load_bedgraph2d(filename, bin_size=None):
+def load_bedgraph2d(filename, bin_size=None, fragments_file=None):
     """
     Loads matrix and fragment information from a 2D bedgraph file. Note this
     function assumes chromosomes are ordered in alphabetical. order
@@ -508,6 +688,9 @@ def load_bedgraph2d(filename, bin_size=None):
         Path to the bedgraph2D file.
     bin_size : int
         The size of bins in the case of fixed bin size.
+    fragments_file : str
+        Path to a fragments file to explicitely provide fragments positions.
+        If the matrix does not have fixed bin size, this prevents errors.
     
     Returns
     -------
@@ -518,21 +701,20 @@ def load_bedgraph2d(filename, bin_size=None):
         The list of fragments/bin present in the matrix with their genomic
         positions.
     """
-    bed2d = pd.read_csv(filename, sep=" ", header=None)
-    if bin_size is None:
+    bed2d = pd.read_csv(filename, sep="\t", header=None)
+    chrom_sizes = {}
+    if bin_size is not None:
+        # If bin size if provided, retrieve chromosome lengths, this will be
+        # used when regenerating bin coordinates
+        chroms = bed2d[[0, 2]].groupby([0], sort=False).max()
+        for chrom, size in zip(chroms.index, np.array(chroms)):
+            chrom_sizes[chrom] = size[0]
+    elif fragments_file is None:
         logger.warning(
             "Please be aware that not all information can be restored from a "
             "cooler file without fixed bin size; fragments without any contact "
             "will be lost"
         )
-    else:
-        # If bin size if provided, retrieve chromosome lengths, this will be
-        # used when regenerating bin coordinates
-        chroms = bed2d[[0, 2]].groupby([0], sort=False).max()
-        chrom_sizes = {}
-        for chrom, size in zip(chroms.index, np.array(chroms)):
-            chrom_sizes[chrom] = size[0]
-
     # Get all possible fragment chrom-positions into an array
     frag_pos = np.vstack([np.array(bed2d[[0, 1]]), np.array(bed2d[[3, 4]])])
     # Sort by position (least important, col 1)
@@ -552,17 +734,24 @@ def load_bedgraph2d(filename, bin_size=None):
     frag_pos_b = np.array(
         bed2d[[3, 4]].apply(lambda x: "".join(x.astype(str)), axis=1).tolist()
     )
-    if bin_size is None:
-        frag_map = {v: i for i, v in enumerate(ordered_frag_pos)}
-    else:
-        # If fixed fragment size available, use it to reconstruct original
-        # fragments ID (even if they are absent from the bedgraph file).
+    # If fragments file is provided, use fragments positions to indices mapping
+    if fragments_file is not None:
+        frags = pd.read_csv(fragments_file, delimiter="\t")
+        frag_map = frags.apply(
+            lambda x: "".join([str(x.chrom), str(x.start_pos)]), axis=1
+        )
+        frag_map = {f_name: f_idx for f_idx, f_name in enumerate(frag_map)}
+    # If fixed fragment size available, use it to reconstruct original
+    # fragments ID (even if they are absent from the bedgraph file).
+    elif bin_size is not None:
         frag_map = {}
         for chrom, size in chrom_sizes.items():
             prev_frags = len(frag_map)
             for bin_id, bin_pos in enumerate(range(0, size, bin_size)):
                 frag_map["".join([chrom, str(bin_pos)])] = bin_id + prev_frags
-
+    # If None available, guess fragments indices from bedgraph (potentially wrong)
+    else:
+        frag_map = {v: i for i, v in enumerate(ordered_frag_pos)}
     # Match bin indices to their names
     frag_id_a = np.array(list(map(lambda x: frag_map[x], frag_pos_a)))
     frag_id_b = np.array(list(map(lambda x: frag_map[x], frag_pos_b)))
@@ -592,6 +781,7 @@ def save_bedgraph2d(mat, frags, out_path):
         correspond to the content of the fragments_list.txt file.
 
     """
+
     mat_df = pd.DataFrame(
         {"row": mat.row, "col": mat.col, "data": mat.data.astype(int)}
     )
@@ -614,7 +804,7 @@ def save_bedgraph2d(mat, frags, out_path):
     )
     # Select only relevant columns in correct order
     bg2 = merge_mat.loc[:, ["chr1", "start1", "end1", "chr2", "start2", "end2", "data"]]
-    bg2.to_csv(out_path, header=None, index=False, sep=" ")
+    bg2.to_csv(out_path, header=None, index=False, sep="\t")
 
 
 def sort_pairs(in_file, out_file, keys, tmp_dir=None, threads=1, buffer="2G"):
@@ -641,11 +831,27 @@ def sort_pairs(in_file, out_file, keys, tmp_dir=None, threads=1, buffer="2G"):
     """
     # TODO: Write a pure python implementation to drop GNU coreutils depencency,
     # could be inspired from: https://stackoverflow.com/q/14465154/8440675
+
+    # Check if UNIX sort version supports parallelism
+    parallel_ok = True
+    sort_ver = sp.Popen(["sort", "--version"], stdout=sp.PIPE)
+    sort_ver = (
+        sort_ver.communicate()[0].decode().split("\n")[0].split(" ")[-1].split(".")
+    )
+    sort_ver = list(map(int, sort_ver))
+    # If so, specify threads, otherwise don't mention it in the command line
+    if sort_ver[0] < 8 or (sort_ver[0] == 8 and sort_ver[1] < 23):
+        logger.warning(
+            "GNU sort version is {0} but >8.23 is required for parallel "
+            "sort. Sorting on a single thread.".format(".".join(map(str, sort_ver)))
+        )
+        parallel_ok = False
+
     key_map = {
         "readID": "-k1,1d",
-        "chr1": "-k2,2d",
+        "chr1": "-k2,2V",
         "pos1": "-k3,3n",
-        "chr2": "-k4,4d",
+        "chr2": "-k4,4V",
         "pos2": "-k5,5n",
         "strand1": "-k6,6d",
         "strand2": "-k7,7d",
@@ -659,7 +865,6 @@ def sort_pairs(in_file, out_file, keys, tmp_dir=None, threads=1, buffer="2G"):
     except KeyError:
         print("Unkown column name.")
         raise
-
     # Rewrite header with new sorting order
     header = get_pairs_header(in_file)
     with open(out_file, "w") as output:
@@ -671,13 +876,12 @@ def sort_pairs(in_file, out_file, keys, tmp_dir=None, threads=1, buffer="2G"):
 
     # Sort pairs and append to file.
     with open(out_file, "a") as output:
-        grep_cmd = sp.Popen(["grep", "-v", "^#", in_file], stdout=sp.PIPE)
-        sort_cmd = sp.Popen(
-            ["sort", "--parallel=%d" % threads, "-S %s" % buffer] + list(sort_keys),
-            stdin=grep_cmd.stdout,
-            stdout=output,
-        )
-        sort_cmd.communicate()
+        grep_proc = sp.Popen(["grep", "-v", "^#", in_file], stdout=sp.PIPE)
+        sort_cmd = ["sort", "-S %s" % buffer] + list(sort_keys)
+        if parallel_ok:
+            sort_cmd.append("--parallel={0}".format(threads))
+        sort_proc = sp.Popen(sort_cmd, stdin=grep_proc.stdout, stdout=output)
+        sort_proc.communicate()
 
 
 def get_pairs_header(pairs):
@@ -717,3 +921,69 @@ def get_pairs_header(pairs):
             line = pairs.readline()
 
     return header
+
+
+def reorder_fasta(genome, output=None, threshold=100000):
+    """Reorder and trim a fasta file
+
+    Sort a fasta file by record lengths, optionally trimming the smallest ones.
+
+
+    Parameters
+    ----------
+    genome : str, file or pathlib.Path
+        The genome scaffold file (or file handle)
+    output : str, file or pathlib.Path
+        The output file to write to
+    threshold : int, optional
+        The size below which scaffolds are discarded, by default 100000
+    """
+
+    if output is None:
+        output = "{}_renamed.fa".format(genome.split(".")[0])
+
+    handle = SeqIO.parse(genome, "fasta")
+    handle_to_write = sorted(
+        (len(u) for u in handle if len(u) > threshold), reverse=True
+    )
+    SeqIO.write(handle_to_write, output, "fasta")
+
+
+def rename_genome(genome, output=None, ambiguous=True):
+    """Rename genome and slugify headers
+
+    Rename genomes according to a simple naming scheme; this
+    is mainly done to avoid special character weirdness.
+
+    Parameters
+    ----------
+    genome : file, str or pathlib.Path
+        The input genome to be renamed and slugify.
+    output : file, str or pathlib.Path
+        The output genome to be written into. Default is <base>_renamed.fa,
+        where <base> is genome_in without its extension.
+    ambiguous : bool
+        Whether to retain ambiguous non-N bases, otherwise rename them to Ns.
+        Default is True.
+    """
+
+    if output is None:
+        output = "{}_renamed.fa".format(genome.split(".")[0])
+
+    with open(output, "w") as output_handle:
+        for record in SeqIO.parse(genome, "fasta"):
+
+            # Replace hyphens, tabs and whitespace with underscores
+            new_record_id = record.id.replace(" ", "_")
+            new_record_id = new_record_id.replace("-", "_")
+            new_record_id = new_record_id.replace("\t", "_")
+
+            # Remove anything that's weird, i.e. not alphanumeric
+            # or an underscore
+            new_record_id = re.sub("[^_A-Za-z0-9]+", "", new_record_id)
+            header = ">{}\n".format(new_record_id)
+
+            new_seq = re.sub("[^ATGCatgcNn]", "N", str(record.seq))
+
+            output_handle.write(header)
+            output_handle.write("{}\n".format(new_seq))
